@@ -1,13 +1,51 @@
 import os
+import shutil
 import subprocess
 from dataclasses import dataclass
 from pathlib import Path
 from typing import List, Optional
 from urllib.parse import urlparse
+from warnings import warn
 
 from dataclasses_json import dataclass_json
 
 
+def run(cmd: List[str]):
+    full_command = " ".join(cmd)
+    print(f"Running '{full_command}'")
+    subprocess.run(cmd, check=True)
+
+
+class LGenomeBackend:
+    def cp(self, src: str, dst: str, *, show_progress: bool = True):
+        raise NotImplementedError()
+
+    def sync(self, src: str, dst: str, *, show_progress: bool = True):
+        raise NotImplementedError()
+
+
+class LGenomeS5cmdBackend(LGenomeBackend):
+    def cp(self, src: str, dst: str, *, show_progress: bool = True):
+        run(["s5cmd", "cp", src, dst])
+
+    def sync(self, src: str, dst: str, *, show_progress: bool = True):
+        run(["s5cmd", "sync", src + "*", dst])
+
+
+class LGenomeS3Backend(LGenomeBackend):
+    def cp(self, src: str, dst: str, *, show_progress: bool = True):
+        cmd = ["aws", "s3", "cp", src, dst]
+        if not show_progress:
+            cmd.append("--no-progress")
+        run(cmd)
+
+    def sync(self, src: str, dst: str, *, show_progress: bool = True):
+        cmd = ["aws", "s3", "sync", src, dst]
+        if not show_progress:
+            cmd.append("--no-progress")
+        run(cmd)
+
+
 @dataclass_json
 @dataclass
 class GenomeData:
@@ -17,10 +55,6 @@ class GenomeData:
     salmon_index: Optional[str]
 
 
-def run(cmd: List[str]):
-    subprocess.run(cmd, check=True)
-
-
 GenomeRegistry = {
     "RefSeq_hg38_p14": GenomeData(
         gtf="s3://latch-genomes/Homo_sapiens/RefSeq/GRCh38.p14/GCF_000001405.40_GRCh38.p14_genomic.stripped.gtf",
@@ -58,11 +92,23 @@ class NoGenomeResourceFoundException(Exception):
 
 
 class GenomeManager:
-    def __init__(self, gid: str):
+    def __init__(self, gid: str, *, backend: Optional[LGenomeBackend] = None):
 
         # TODO (kenny) check valid gids.
         self._gid = gid
 
+        self.backend = None
+        if self.backend is None:
+            if shutil.which("s5cmd") is not None:
+                self.backend = LGenomeS5cmdBackend()
+            if shutil.which("s3") is not None:
+                self.backend = LGenomeS3Backend()
+
+        if self.backend is None:
+            warn(
+                "No available lgenome backend - install an appropriate blobstore CLI (eg. s3 or s5cmd.)"
+            )
+
     def get_genome_data(self) -> GenomeData:
 
         g_data = GenomeRegistry.get(self._gid)
@@ -72,7 +118,7 @@ class GenomeManager:
             )
         return g_data
 
-    def download_gtf(self) -> Path:
+    def download_gtf(self, show_progress: bool = False) -> Path:
 
         g_data = self.get_genome_data()
 
@@ -84,15 +130,8 @@ class GenomeManager:
 
         local_gtf = Path.cwd() / Path(urlparse(gtf).path).name
 
-        run(
-            [
-                "aws",
-                "s3",
-                "cp",
-                gtf,
-                str(local_gtf.resolve()),
-            ]
-        )
+        assert self.backend is not None
+        self.backend.cp(gtf, str(local_gtf.resolve()), show_progress=show_progress)
 
         return local_gtf
 
@@ -108,15 +147,8 @@ class GenomeManager:
 
         local_ref_genome = Path.cwd() / Path(urlparse(ref_genome).path).name
 
-        run(
-            [
-                "aws",
-                "s3",
-                "cp",
-                ref_genome,
-                str(local_ref_genome.resolve()),
-            ]
-        )
+        assert self.backend is not None
+        self.backend.cp(ref_genome, str(local_ref_genome.resolve()))
 
         return local_ref_genome
 
@@ -132,19 +164,12 @@ class GenomeManager:
 
         local_ref_trans = Path.cwd() / Path(urlparse(ref_trans).path).name
 
-        run(
-            [
-                "aws",
-                "s3",
-                "cp",
-                ref_trans,
-                str(local_ref_trans.resolve()),
-            ]
-        )
+        assert self.backend is not None
+        self.backend.cp(ref_trans, str(local_ref_trans.resolve()))
 
         return local_ref_trans
 
-    def download_salmon_index(self) -> Path:
+    def download_salmon_index(self, show_progress: bool = True) -> Path:
 
         g_data = self.get_genome_data()
 
@@ -156,14 +181,10 @@ class GenomeManager:
 
         os.mkdir("salmon_index")
         local_salmon_index = Path("salmon_index").resolve()
-        run(
-            [
-                "aws",
-                "s3",
-                "sync",
-                salmon_index,
-                str(local_salmon_index),
-            ]
+
+        assert self.backend is not None
+        self.backend.sync(
+            salmon_index, str(local_salmon_index), show_progress=show_progress
         )
 
         return local_salmon_index
